----------------------------------------------------------------------------------
-- VGA_Driver
-- Basically two counters VCounter:HCounter - which used to generate
-- The correct Sync signals.
-- This interface adjusted for the 240x256 VideoRAM in order to generate a
-- 480x640 (Lines x Pixel) The first and last 64 pixel at each line will be blanked
-- Hence will the 256 pixel informations fill 256x2=512 pixels
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity VGA_Driver is
Port (CLK : 	in  std_logic;
		RESET: 	in  std_logic;
		HSYNCH:  out std_logic;
		VSYNCH:	out std_logic;
		HBLANK: 	out std_logic;
		Adr:     out std_logic_vector(15 downto 0) --Line(7:0) + Pixel(7:0) 
		);
end VGA_Driver;

architecture Behavioral of VGA_Driver is
   signal LINE: 	 	 std_logic_vector( 7 downto 0);
   signal PIXEL: 		 std_logic_vector( 7 downto 0);
	signal VCLK:          std_logic;
begin

   Adr <= LINE & PIXEL;
-----------------------------------------------------------------------
-- Horizontal State Machine
-----------------------------------------------------------------------
Horizontal: process ( CLK, RESET )
	type HSTATE is ( 	FRONT_PORCH, 
							SYNCH,           --  96 pixels (clk-pulses) wide
							BACK_PORCH, 
							LEFT_BORDER, 
							ACTIVE_VIDEO_1,    -- 64 pixels
							ACTIVE_VIDEO_2,    -- 512 pixels (actually 256)
							ACTIVE_VIDEO_3,    -- 64 pixels
							RIGHT_BORDER );
	variable Horizontal_State : HSTATE := LEFT_BORDER;
	variable HCounter:     integer range 0 to 1023 := 0;
	variable Scale5:       integer range 0 to 7    := 0;
	variable Scale2:       integer range 0 to 2    := 0;
begin
	if ( RESET = '1' ) then
		Horizontal_State 	:= FRONT_PORCH;
	   HCounter         	:= 0;
		HSYNCH 				<= '1';
		VCLK 					<= '0';
		HBLANK 				<= '1';
	elsif ( CLK = '1' and CLK'EVENT ) then
	   Scale2 := Scale2+1;
		if Scale2=2 then
			Scale2 := 0;
			VCLK 		<= '0';
			HBLANK 	<= '1';
			HCounter := HCounter+1;    
			HSYNCH 	<= '1';		
	
			case ( Horizontal_State ) is
				--------------------------- 5 pixels----- Want Left Border		
				when LEFT_BORDER 	=>
					if (HCounter >= 5) then				
						Horizontal_State 	:= ACTIVE_VIDEO_1;
						HCounter      		:= 0;
						PIXEL            	<= (others=>'0');
						Scale5           	:= 0;
					end if;
				------------------------------------- Want 64 Blank Pixels
				when ACTIVE_VIDEO_1 =>		
					HBLANK <= '1';
					if (HCounter >= 64) then			 
						Horizontal_State 	:= ACTIVE_VIDEO_2;
					end if;
				------------------------------------- Want 512 Active Pixels
				when ACTIVE_VIDEO_2 =>		
					HBLANK <= '0';
					Scale5:=Scale5+1;
					IF Scale5=2 then
						Scale5 := 0; PIXEL <= PIXEL+1;
					END IF;
					if (HCounter >= (640-64)) then			 
						Horizontal_State 	:= ACTIVE_VIDEO_3;
					end if;
				------------------------------------- Want 64 blank Pixels
				when ACTIVE_VIDEO_3 =>		
					HBLANK <= '1';
					if (HCounter >= 640) then			 
						Horizontal_State 	:= RIGHT_BORDER;
						HCounter      		:= 0;
					end if;
				------------------------------ 6 ---- Want Right Border Pixels
				when RIGHT_BORDER =>
					if (HCounter >= 6) then	--6
						Horizontal_State 	:= FRONT_PORCH;
						HCounter      		:= 0;
					end if;
				---------------------------- 7+9=16 -- Want Front porch Pixels
				when FRONT_PORCH =>
					if (HCounter >= 16) then				
						Horizontal_State 	:= SYNCH;
						HCounter      		:= 0;
						VCLK             	<= '1';        -- Generate a VCLK __---
					end if;
				-------------------------------- 96 pixels -- Wanted Synch puls
				when SYNCH =>
					HSYNCH <= '0';
					if (HCounter >= 96) then				 
						Horizontal_State 	:= BACK_PORCH;
						HCounter      		:= 0;
					end if;
				----------------------------- 47 pixels ----- Wanted Back Porch
				-- This value will move the sceen image vertically 		
				when BACK_PORCH =>
					if (HCounter >= 47) then		--47	
						Horizontal_State 	:= LEFT_BORDER;
						HCounter      		:= 0;
					end if;
				when others =>
					Horizontal_State 	:= LEFT_BORDER;
					HCounter      		:= 0;
					HSYNCH          	<= '1';
			end case;
		end if;
	end if;
end process;

-----------------------------------------------------------------------
-- The Vertical State Machine
-- Please note - still some timing problems - adjustments needed 
-----------------------------------------------------------------------
Vertical: process ( VCLK, RESET )
	type VSTATE is ( FRONT_PORCH, 
						  SYNCH,         --   2 lines
						  BACK_PORCH, 
						  TOP_BORDER, 
						  ACTIVE_VIDEO,  -- 480 lines
						  BOTTOM_BORDER );
	variable Vertical_State : VSTATE := TOP_BORDER;
	variable VCounter :     integer range 0 to 1023 := 0;
	variable TEMP_LINE:     std_logic_vector( 8 downto 0);
begin
	if (RESET = '1' ) then
		Vertical_State 			:= TOP_BORDER;	
		VCounter                := 0;
		VSYNCH 						<= '1';
	elsif ( VCLK = '1' and VCLK'EVENT ) then	    
		VSYNCH 	 <= '1';
		VCounter  := VCounter+1;
		case ( Vertical_State ) is
			--------------------6+2=8 lines-- Want a Vertical Front porch 
			when FRONT_PORCH =>
				if (VCounter = 2) then			
					Vertical_State := SYNCH;
					VCounter       := 0;
				end if;
			-----------------------------2 lines--- Want a Vertical Synch 
			when SYNCH =>
				VSYNCH <= '0';
				if (VCounter >= 2) then				 
					Vertical_State := BACK_PORCH;
					VCounter       := 0;
				end if;
			---------------------------24----- Want a Vertical Back Porch 
			when BACK_PORCH =>
				if (VCounter >= 17) then				
					Vertical_State := TOP_BORDER;
					VCounter       := 0;
				end if;
         ---------------------24+5=29 lines-- Want a Vertical Top border
			when TOP_BORDER 	=>
				if (VCounter >= 5) then			
					Vertical_State := ACTIVE_VIDEO;
					VCounter       := 0;
					LINE				<= ( others => '0' );
				end if;
			--------------------------------------------- Want 480 lines
			when ACTIVE_VIDEO =>
				TEMP_LINE := CONV_STD_LOGIC_VECTOR(VCounter,9);
				LINE      <= TEMP_LINE(8 downto 1);
				if (VCounter >= 480) then			 
					Vertical_State := BOTTOM_BORDER;
					VCounter       := 0;
				end if;
			-------------------------------6------ Want a Vertical border
			when BOTTOM_BORDER =>
				if (VCounter >= 6) then			
					Vertical_State := FRONT_PORCH;
					VCounter       := 0;
				end if;
			when others =>
				Vertical_State := FRONT_PORCH;
				VCounter       := 0;
				VSYNCH 			<= '1';
			end case;
	end if;
end process;

end Behavioral;

